new;

new;

/*************************************************
 Chernozhukov, Lee, and Rosen (2012)
 - Replication File for Generating Figures 3 and 4
 - Empircal Illustration based on Manski and Pepper (2000) 
 - Adding 20 bootstrapped values of the original estimated curve to the figure 
***************************************************/

library pgraph;
output file = clr_example_series.out reset;
output off;


format /rd /m1 12,2;



/********************
*** Loading Data ****
********************/


n_all = 2044;
d_all = 4;  
   yr = 1994;
   
load data[n_all,d_all] = clr_nlsy_example.dat;


/***************************************************************
DATA SOURCE: NLSY 
DATA CITATION: The sample used here is from the following paper. 
Carneiro, P., and S. Lee (2009): Estimating Distributions of Potential Outcomes Using Local Instrumental
Variables with an Application to Changes in College Enrollment and Wage Inequality, Journal of Econometrics, 149, 191.208.
****************************************************************
 Year: 1994 (same as Manski and Pepper, 2000)
 Sample Size: 2044
 Data: Column (1): Log hourly wages in 1994 (averaged over neighboring 5 years)
       Column (2): Years of schooling
       Column (3): Education achievement
                  (1: high school dropouts; 2: high school graduates;
                   3: some college; 4: college graduates and more)
       Column (4): AFQT (schooling-adjusted)
****************************************************************/


 data = sortc(data,4);              @ sort based on AFQT @
    n = rows(data);
    
    y = data[.,1];
    z = data[.,2];
    v = data[.,4];

y_value = ln(24);

tmat = {16};
 i_t = 1;
do until i_t > rows(tmat); 

    t = tmat[i_t];                  @ treatment: 16 years of education @
   v0 = 0;                          @ average AFQT person @

   yu = (y .> y_value).*(z .>= t) + (z .< t);   @ outcome of interest Pr[ y > y_value | v0 ] @ 



/**************************
*** Tuning Parameters  ****
**************************/


         nn = n;         
     vsemtd = 1;                   @ no argmin set estimation @
   cvmethod = 1;                   @ method for choosing critical values: simulation method @ 
   semethod = 1;                   @ heteroskedastic conditional variance function @




/**************************
***    Simulation      ****
**************************/


    seed = 2545345;

      RR = 10000;
   rnd_R = rndns(101,RR,seed);   @ random numbers that will be used to callibrate critical values @
  
    

@ Smoothing parameter selection by leave-one-out cross validation @
      

nf_x_set_terms = 5 * ceil(nn^(1/5));
nf_x_set = seqa(5,1,nf_x_set_terms);
cvmat = {};
nf_x_checked = {};
i_nf_x = 1;

@ cross validate over all nf_x in nf_x set @
@ If greatest nf_x checked selected, then keep searching @
do until (i_nf_x > rows(nf_x_set)) AND (minindc(cvmat) < (i_nf_x-1));

nf_x_i = minc(nf_x_set) + i_nf_x - 1;     
 
@ B-splines @  

	intercept = 1;
     xmin = minc(v);
      xmax = maxc(v);
    knstep = (nf_x_i-4);                                   @ number of interior knots @
     knq_x = seqa(1/(knstep+1), 1/(knstep+1), knstep);     
      kt_x = quantile(v,knq_x);                            @ equi-spaced in terms of sample quantiles @
     psi_x = B_splines(v,3,kt_x,xmin,xmax,intercept);      @ cubic splines with the intercept @


@ ***** Sieve Estimation ***** @

      bhat = invpd(psi_x'psi_x)*psi_x'yu;
      uhat = yu - psi_x*bhat;
     L_mat = psi_x*invpd(psi_x'psi_x)*psi_x';
      rhat = uhat./(1-diag(L_mat));
     cvmat = cvmat|(rhat'rhat)/rows(rhat);
	 nf_x_checked = nf_x_checked|nf_x_i;

    i_nf_x = i_nf_x + 1;
endo;

   nf_x = nf_x_checked[minindc(cvmat)];

   
      nf_x = nf_x*nn^(-1/5)*nn^(2/7);
      nf_x = floor(nf_x);   @ undersmoothing @



  


@ argmax set estimation @

   sp_large = v0~2;    @ support of v for which the minimum is taken @

if vsemtd == 1; @ use the whole support @

      vseq = 100;
 argminset = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
      sp_l = sp_large;
      


elseif vsemtd == 2; @ use an estimated set @


@-- estimation of the argmin set --@

           vseq = 100;
       vs_large = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
         gamma_n = 0.1/ln(nn);

  {ml, sel, cvl} = sieve_est(yu,v,vs_large,nf_x,gamma_n); 

          cutoff = minc(ml + sel*cvl) + 2*sel*cvl;
       argminset = selif(vs_large,(ml .<= cutoff));  @ estimated argminset @
            sp_l = minc(argminset)~maxc(argminset);

endif;







@ Bounds estimatin: MM (mean monotonicity, or MIV) @


@--- Bounds estimation ---@


levels = 0.50|0.05;

{ml2, sel2, cvl2} = sieve_est(yu,v,argminset,nf_x,levels);    

  u_b = minc(ml2);

@--- Asymptotically median unbiased estimation  ---@

 ep_l = ml2 + sel2*cvl2[1];
 u_b1 = minc(ep_l);


@--- Constructing confidence regions ---@

 ep_l = ml2 + sel2*cvl2[2];
 u_b2 = minc(ep_l);


@--- collecting outputs ---@

ubb = u_b|u_b1|u_b2;





/*********************************
***    Figure 4 in the paper  ****
*********************************/
 
if t == 16;

graphset;
_pdate="";
_plegctl = -1;
_pltype = 3|6|1;
_plwidth = 15;
_pcolor = 1|2|3;

_plegctl = {2 5 1 1};

_plegstr = "Estimated Curve\000Precision-Corrected Curve\000Endpont of 95\% One-Sided C. I.";


xlabel(" AFQT ");
ylabel(" Probability  ");

xtics(0,2,0.5,0);

ytics(0.2,1,0.2,2); 

_ptek="nlsy-fig.tkf";

title(" MIV-MTR Upper Bounds ");



xy(argminset,ml2~ep_l~u_b2*ones(rows(argminset),1));


/*** Converting graphs to ps files ***/

ret=tkf2ps("nlsy-fig.tkf","nlsy-fig.ps");

endif;





/**************************
*** Summary Statistics ****
**************************/

  sum_data = data;
    

output on;
"============================================================================";
" Estimation of Pr[Y(t) > y_value|V=v0] ";
" y_value: " exp(y_value);
" t:  " t;
" v0: " v0;
" Year: " yr;
"============================================================================";
" Number of Observations  " n;
"";
" Summary of Data:  Mean -- Median -- Stdc -- Min -- Max ";
meanc(sum_data)~median(sum_data)~stdc(sum_data)~minc(sum_data)~maxc(sum_data);
"============================================================================";
" Smoothing parameter " nf_x;
" Naive Sample Analog Estimate --- New Estimate --- Confidence Region ";
" upper bound for E[y(t)|v=0] " ubb';
output off;
 



/***********************************
*** Generating Bootrap Samples  ****
***********************************/

rep_star = 20;

result_star = {};

i_star = 1;
do until i_star > rep_star;


       bbsample = ceil(n*rndus(n,1,seed));          @ random sampling with replacement @
         y_star = y[bbsample];
         z_star = z[bbsample];
         v_star = v[bbsample];

        yu_star = (y_star .> y_value).*(z_star .>= t) + (z_star .< t);   @ outcome of interest Pr[ y > y_value | v0 ] @ 


@ Use the same smoothing parameter as above @
      


@ argmax set estimation @

   sp_large = v0~2;    @ support of v for which the minimum is taken @

if vsemtd == 1; @ use the whole support @

      vseq = 100;
 argminset = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
      sp_l = sp_large;
      


elseif vsemtd == 2; @ use an estimated set @


@-- estimation of the argmin set --@

           vseq = 100;
       vs_large = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
         gamma_n = 0.1/ln(nn);

  {ml, sel, cvl} = sieve_est(yu_star,v_star,vs_large,nf_x,gamma_n); 

          cutoff = minc(ml + sel*cvl) + 2*sel*cvl;
       argminset = selif(vs_large,(ml .<= cutoff));  @ estimated argminset @
            sp_l = minc(argminset)~maxc(argminset);

endif;




@ Bounds estimatin: MM (mean monotonicity, or MIV) @


@--- Bounds estimation ---@


{ml2_star, sel2_star, cvl2_star} = sieve_est(yu_star,v_star,argminset,nf_x,levels);    

result_star = result_star~ml2_star;

i_star = i_star + 1;
endo;


/***********************************
***     Figure 3 in the paper   ****
***********************************/
 
if t == 16;

graphset;
_pdate="";
_plegctl = -1;
_pltype = 6|3*ones(rep_star,1);
_plwidth = 15|5*ones(rep_star,1);
_pcolor = 2|1*ones(rep_star,1);


xlabel(" AFQT ");
ylabel(" Probability  ");

xtics(0,2,0.5,0);
@
ytics(0.2,1,0.2,2); 
@
_ptek="nlsy-fig-b.tkf";

title(" MIV-MTR Upper Bounds ");

xy(argminset,ml2~result_star);



/*** Converting graphs to ps files ***/

ret=tkf2ps("nlsy-fig-b.tkf","nlsy-fig-b.ps");

endif;












i_t = i_t + 1;
endo;
 
end;

/*********************************************************/
/*                Procedures                             */
/*********************************************************/



/**********************************************************/
/******           B-splines                         *******/
/**********************************************************/


proc(1) = B_splines(x,deg,knots,xmin,xmax,intercept);
@---
Input:
    x = (n*1) vector of data
  deg = degree (order = df + 1)
knots = interior knots
 xmin = lower bound
 xmax = upper bound
intercept = 1 (intercept included); = 0 (intercept excluded)
Output:
(n*kappa) matrix of B-spline basis, where kappa = #(knots)+order.
Reference:
A Practical Guide to Splines by Carl De Boor (Springer, 1978)
Note:
This gives the same output as command "bs" in the splines package of R.
---@

local k,t,tr1,tr2,m1tmp,j,tr3,tr4,m11,m12,term1,term2,den1,den2;

       k = deg + 1;                               @ order @
       t = xmin|knots|xmax;                       @ an initial knot sequence @
     tr1 = trimr(t,0,1);                          @ a vector of t[i]         @
     tr2 = trimr(t,1,0);                          @ a vector of t[i+1]       @
   m1tmp = (x .>= tr1').*(x .< tr2');             @ B-splines of degree 0    @


if k > 1;

j = 2;
do until j > k;

      t = xmin*ones(j,1)|knots|xmax*ones(j,1);               @ an updated knot sequence @
  m1tmp = zeros(rows(m1tmp),1)~m1tmp~zeros(rows(m1tmp),j);

    tr1 = trimr(t,0,j);
    tr2 = trimr(t,j-1,1);
    m11 = trimr(m1tmp',0,j)';
   den1 = (tr2 - tr1)';
   den1 = (1-(den1.==0))./(den1 + (den1.==0));               @ 0/0 = 0 convention @
  term1 = (x-tr1').*m11.*den1;

    tr3 = trimr(t,j,0);
    tr4 = trimr(t,1,j-1);
    m12 = trimr(m1tmp',1,j-1)';
   den2 = (tr3 - tr4)';
   den2 = (1-(den2.==0))./(den2 + (den2.==0));
  term2 = (tr3'-x).*m12.*den2;

  m1tmp = term1 + term2;                                     @ recursive formula @

j = j + 1;
endo;

endif;

if intercept == 0;
m1tmp = m1tmp[.,2:cols(m1tmp)];
endif;

retp(m1tmp);

endp;

/*******************************************************/
/*           Sieve estimation                          */
/*******************************************************/

proc(3) = sieve_est(y,x,z,nf_x,level);
@---
Input:
          y,x = data
            z = points at which a function is estimated 
         nf_x = numbers of approximating functions   
        level = level of the confindence set

Output: function estimates, their standard errors, uniform critical values
---@

local n,intercept,xmin,xmax,knstep,knq_x,kt_x,psi_x,psi_z,tmp1,tmp2,bhat,ghat,uhat,tmp3,var,varhat,sehat,
      Sigmahat, RR, rnd_RR, tmp_RR, acv; 

@ ***** Basis functions ***** @

n = rows(y);

@ B-splines @  

 intercept = 1;
      xmin = minc(x);
      xmax = maxc(x);
    knstep = (nf_x-4);                                     @ number of interior knots @
     knq_x = seqa(1/(knstep+1), 1/(knstep+1), knstep);     
      kt_x = quantile(x,knq_x);                            @ equi-spaced in terms of sample quantiles @
     psi_x = B_splines(x,3,kt_x,xmin,xmax,intercept);      @ cubic splines with the intercept @

     psi_z = B_splines(z,3,kt_x,xmin,xmax,intercept);  

@ ***** Sieve Estimation ***** @

   tmp1 = invpd(psi_x'psi_x);
   tmp2 = psi_x';
   bhat = tmp1*(tmp2*y);
   ghat = psi_z*bhat;
   
   uhat = y - psi_x*bhat;
   tmp3 = tmp2*diagrv(eye(n),uhat^2)*tmp2';
    var = tmp1*tmp3*tmp1';
 varhat = psi_z*var*psi_z';
  sehat = sqrt(diag(varhat));
  
      Sigmahat = diagrv(eye(rows(sehat)),1/sehat)*varhat*diagrv(eye(rows(sehat)),1/sehat);


            RR = 10000;
        rnd_RR = chol(Sigmahat + 1e-8*eye(rows(Sigmahat)))'rnd_R[1:rows(z),.];    @ rnd_R is defined in the main program @
        tmp_RR = maxc(rnd_RR);

    
    
/****** Uniform confindence intervals *******/


  acv = quantile(tmp_RR,1-level);  @ asymptotic critical values @ 
  
  
retp(ghat, sehat, acv);
endp;

