new;

/*************************************************
 Chernozhukov, Lee, and Rosen (2012)
 Monte Carlo Simulations for DGPs 1-4
 B-spline Series Estimation
 
 Simulation design 

 Y_i =  L*\phi(x) + C* e_i,   x \in [-2,2]

four cases: L = 0 --> flat case;  L=1 --> modest flat; L=5 --> peaked

 \theta_0 = max L*\phi(x)

 estimation of the set V_0 
 series estimation
***************************************************/


output file = clr_mc_Bspline_series_DGP_1_4.out reset;

output on;
"======== Monte Carlo Results ================================================";
"Column 1: Data generation process ";
"Column 2: Sample size ";
"Column 3: Average K_n ";
"Column 4: argmin set estimation method ";
"Column 5: Coverage Probability (95%) ";
"Column 6: False Coverage Probability (95%) ";
"Column 7: Average argmin set lower bound ";
"Column 8: Average argmin set upper bound ";
"============================================================================";
output off;




format /rd /m1 7,3;

tstart = date;


dgp_set = {1,2,3,4};
  n_set = {500,1000};
cvm_set = {1};
vse_set = {1,2};


i_dgp = 1;
do until i_dgp > rows(dgp_set);

i_n = 1;
do until i_n > rows(n_set); 

i_vse = 1;                      
do until i_vse > rows(vse_set); 

i_cvm = 1;
do until i_cvm > rows(cvm_set);

nf_cv_max_count = 0;

/**************************
*** Tuning Parameters  ****
**************************/


         nn = n_set[i_n];          @ sample size in simulations @
        dgp = dgp_set[i_dgp];      @ DGP @ 
     vsemtd = vse_set[i_vse];      @ method for argmin set estimation @ 

              

      b_cns = 3;
 


   cvmethod = cvm_set[i_cvm];      @ method for choosing critical values: simulation method @ 

   semethod = 1;                   @ heteroskedastic conditional variance functin @

/*********************************
*** Lower bound to be covered ****
*********************************/


if dgp     == 1; true_lb = 0;  
elseif dgp == 2; true_lb = pdfn(0);  
elseif dgp == 3; true_lb = 5*pdfn(0);  
elseif dgp == 4; true_lb = 5*pdfn(0); 
endif;
             
@               
" true_lb " true_lb;
@





/**************************
***    Simulation      ****
**************************/


    seed = 2345243;

      RR = 10000;
   rnd_R = rndns(101,RR,seed);   @ random numbers that will be used to callibrate critical values @
  
    

    nrep = 1000;

  result_l = {};
  h_matrix = {};
set_matrix = {};

i_rep = 1;
do until i_rep > nrep;
 
               v = 4*rndus(nn,1,seed)-2;  @ U[-2,2] @
               v = sortc(v,1);

if     dgp == 1;   sm0 = 0.1;  
elseif dgp == 2;   sm0 = 0.1;
elseif dgp == 3;   sm0 = 0.1;
elseif dgp == 4;   sm0 = 0.01;
endif;


               u = sm0*rndns(nn,1,seed); 

     
               u = -b_cns*(u .< -b_cns) + u.*(abs(u) .<= b_cns) + b_cns*(u .> b_cns);  

               
if     dgp == 1;   gx0 = 0;  
elseif dgp == 2;   gx0 = pdfn(v);
elseif dgp == 3;   gx0 = 5*pdfn(v);
elseif dgp == 4;   gx0 = 5*pdfn(v);
endif;


          y = gx0 + u;





@ Smoothing parameter selection by leave-one-out cross validation @

nf_x_set_terms = 5 * ceil(nn^(1/5));
nf_x_set = seqa(5,1,nf_x_set_terms);
cvmat = {};
nf_x_checked = {};
i_nf_x = 1;

@ cross validate over all nf_x in nf_x set @
@ If greatest nf_x checked selected, then keep searching @
do until (i_nf_x > rows(nf_x_set)) AND (minindc(cvmat) < (i_nf_x-1));

nf_x_i = minc(nf_x_set) + i_nf_x - 1;     
 
@ B-splines @  

	intercept = 1;
     xmin = minc(v);
      xmax = maxc(v);
    knstep = (nf_x_i-4);                                   @ number of interior knots @
     knq_x = seqa(1/(knstep+1), 1/(knstep+1), knstep);     
      kt_x = quantile(v,knq_x);                            @ equi-spaced in terms of sample quantiles @
     psi_x = B_splines(v,3,kt_x,xmin,xmax,intercept);      @ cubic splines with the intercept @


@ ***** Sieve Estimation ***** @

      bhat = invpd(psi_x'psi_x)*psi_x'y;
      uhat = y - psi_x*bhat;
     L_mat = psi_x*invpd(psi_x'psi_x)*psi_x';
      rhat = uhat./(1-diag(L_mat));
     cvmat = cvmat|(rhat'rhat)/rows(rhat);
	 nf_x_checked = nf_x_checked|nf_x_i;

    i_nf_x = i_nf_x + 1;
endo;

   nf_x = nf_x_checked[minindc(cvmat)];
	  
   nf_x = nf_x*nn^(-1/5)*nn^(2/7);
   nf_x = floor(nf_x);   @ undersmoothing @

  h_matrix = h_matrix|nf_x;

@ end of cross validation @


@ argmax set estimation @

   sp_large = quantile(v,0.05)~quantile(v,0.95);    @ support of v for which the maximum is taken @

if vsemtd == 1; @ use the whole support @

      vseq = 100;
 argminset = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
      sp_l = sp_large;


elseif vsemtd == 2; @ use an estimated set @


@-- estimation of the argmin set --@

           vseq = 100;
       vs_large = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
         gamma_n = 0.1/ln(nn);

  {ml, sel, cvl} = sieve_est(y,v,vs_large,nf_x,gamma_n); 

          cutoff = maxc(ml - sel*cvl) - 2*sel*cvl;
       argminset = selif(vs_large,(ml .>= cutoff));  @ estimated argminset @
            sp_l = minc(argminset)~maxc(argminset);

endif;




set_matrix = set_matrix|sp_l;


@--- Bounds estimatin ---@


levels = 0.50|0.05;

{ml2, sel2, cvl2} = sieve_est(y,v,argminset,nf_x,levels);    

  l_b = maxc(ml2);

@--- Asymptotically median unbiased estimation  ---@

 ep_l = ml2 - sel2*cvl2[1];
 l_b1 = maxc(ep_l);


@--- Constructing confidence regions ---@

 ep_l = ml2 - sel2*cvl2[2];
 l_b2 = maxc(ep_l);


@--- collecting outputs ---@

lbb = l_b|l_b1|l_b2;

  result_l = result_l|lbb';


cov_l = meanc(result_l[.,3] .<= true_lb);

i_rep = i_rep + 1;
endo;


   
l1 = result_l[.,1];
l2 = result_l[.,2];
l3 = result_l[.,3];


      h_ave = meanc(h_matrix);
   sp_l_ave = meanc(set_matrix)';
   
	
	nf_cv_max_pct = nf_cv_max_count / nrep;

/**************************
*** Inference Results ****
**************************/




if (dgp == 1);

false_lb = true_lb - 0.02;

elseif (dgp == 2);

false_lb = true_lb - 0.02;

elseif (dgp == 3);

false_lb = true_lb - 0.02;

elseif (dgp == 4);

false_lb = true_lb - 0.02;

endif;

 cov_cp = meanc(l3 .<= true_lb);
cov_fcp = meanc(l3 .<= false_lb);


output on;

dgp~nn~h_ave~vsemtd~cov_cp~cov_fcp~sp_l_ave;
output off;

i_cvm = i_cvm + 1;
endo;

i_vse = i_vse + 1;      
endo;  
                   


i_n = i_n + 1;
endo;

i_dgp = i_dgp + 1;
endo;



etime = ethsec(tstart,date)/6000;

output on;
"";
"Running time: " etime;
"Number of experiments: " nrep;
 output off; 
 
 
end;

/*********************************************************/
/*                Procedures                             */
/*********************************************************/



/**********************************************************/
/******           B-splines                         *******/
/**********************************************************/


proc(1) = B_splines(x,deg,knots,xmin,xmax,intercept);
@---
Input:
    x = (n*1) vector of data
  deg = degree (order = df + 1)
knots = interior knots
 xmin = lower bound
 xmax = upper bound
intercept = 1 (intercept included); = 0 (intercept excluded)
Output:
(n*kappa) matrix of B-spline basis, where kappa = #(knots)+order.
Reference:
A Practical Guide to Splines by Carl De Boor (Springer, 1978)
Note:
This gives the same output as command "bs" in the splines package of R.
---@

local k,t,tr1,tr2,m1tmp,j,tr3,tr4,m11,m12,term1,term2,den1,den2;

       k = deg + 1;                               @ order @
       t = xmin|knots|xmax;                       @ an initial knot sequence @
     tr1 = trimr(t,0,1);                          @ a vector of t[i]         @
     tr2 = trimr(t,1,0);                          @ a vector of t[i+1]       @
   m1tmp = (x .>= tr1').*(x .< tr2');             @ B-splines of degree 0    @


if k > 1;

j = 2;
do until j > k;

      t = xmin*ones(j,1)|knots|xmax*ones(j,1);               @ an updated knot sequence @
  m1tmp = zeros(rows(m1tmp),1)~m1tmp~zeros(rows(m1tmp),j);

    tr1 = trimr(t,0,j);
    tr2 = trimr(t,j-1,1);
    m11 = trimr(m1tmp',0,j)';
   den1 = (tr2 - tr1)';
   den1 = (1-(den1.==0))./(den1 + (den1.==0));               @ 0/0 = 0 convention @
  term1 = (x-tr1').*m11.*den1;

    tr3 = trimr(t,j,0);
    tr4 = trimr(t,1,j-1);
    m12 = trimr(m1tmp',1,j-1)';
   den2 = (tr3 - tr4)';
   den2 = (1-(den2.==0))./(den2 + (den2.==0));
  term2 = (tr3'-x).*m12.*den2;

  m1tmp = term1 + term2;                                     @ recursive formula @

j = j + 1;
endo;

endif;

if intercept == 0;
m1tmp = m1tmp[.,2:cols(m1tmp)];
endif;

retp(m1tmp);

endp;

/*******************************************************/
/*           Sieve estimation                          */
/*******************************************************/

proc(3) = sieve_est(y,x,z,nf_x,level);
@---
Input:
          y,x = data
            z = points at which a function is estimated 
         nf_x = numbers of approximating functions   
        level = level of the confindence set

Output: function estimates, their standard errors, uniform critical values
---@

local n,intercept,xmin,xmax,knstep,knq_x,kt_x,psi_x,psi_z,tmp1,tmp2,bhat,ghat,uhat,tmp3,var,varhat,sehat,
      Sigmahat, RR, rnd_RR, tmp_RR, acv; 

@ ***** Basis functions ***** @

n = rows(y);

@ B-splines @  

 intercept = 1;
      xmin = minc(x);
      xmax = maxc(x);
    knstep = (nf_x-4);                                     @ number of interior knots @
     knq_x = seqa(1/(knstep+1), 1/(knstep+1), knstep);     
      kt_x = quantile(x,knq_x);                            @ equi-spaced in terms of sample quantiles @
     psi_x = B_splines(x,3,kt_x,xmin,xmax,intercept);      @ cubic splines with the intercept @

     psi_z = B_splines(z,3,kt_x,xmin,xmax,intercept);  

@ ***** Sieve Estimation ***** @

   tmp1 = invpd(psi_x'psi_x);
   tmp2 = psi_x';
   bhat = tmp1*(tmp2*y);
   ghat = psi_z*bhat;
   
   uhat = y - psi_x*bhat;
   tmp3 = tmp2*diagrv(eye(n),uhat^2)*tmp2';
    var = tmp1*tmp3*tmp1';
 varhat = psi_z*var*psi_z';
  sehat = sqrt(diag(varhat));
  
      Sigmahat = diagrv(eye(rows(sehat)),1/sehat)*varhat*diagrv(eye(rows(sehat)),1/sehat);


            RR = 10000;
        rnd_RR = chol(Sigmahat + 1e-8*eye(rows(Sigmahat)))'rnd_R[1:rows(z),.];    @ rnd_R is defined in the main program @
        tmp_RR = maxc(rnd_RR);

    
    
/****** Uniform confindence intervals *******/


  acv = quantile(tmp_RR,1-level);  @ asymptotic critical values @ 
  
  
retp(ghat, sehat, acv);
endp;
