new;

/*************************************************
 Chernozhukov, Lee, and Rosen (2012)
 Monte Carlo Simulations for DGPs 5-8
 Andrews and Shi (2010) method
 Simulation based on Manski and Pepper (2009)
***************************************************/

output file = clr_mc_AS_DGP_5_8.out reset;
output off;

format /rd /m1 8,3;

tstart = date;

 cp_set = {1,2};   
dgp_set = {1,2,3,4};  
  n_set = {250,500,1000};



output on;
"======== Monte Carlo Results ================================================";
"Column 1: Data generationg process ";
"Column 2: Sample size ";
"Column 3: Coverage Probability (95%) ";
"Column 4: False Coverage Probability (95%) ";
"============================================================================";
output off;



i_dgp = 1;
do until i_dgp > rows(dgp_set);

i_n = 1;
do until i_n > rows(n_set); 


cp_mat = {};


i_cp = 1;
do until i_cp > rows(cp_set);


/**************************
*** Tuning Parameters  ****
**************************/

        dgp = dgp_set[i_dgp];      @ DGP @ 
     cp_tht = cp_set[i_cp];        @ theta to be covered @
         nn = n_set[i_n];          @ sample size in simulations @

      alpha = 0.05;
          t = 1;                         
         v0 = 1.5;                          

     b_cns = 0.5;
         k0 = -b_cns;
         k1 = b_cns; 
        sm0 = 0.1;



/*********************************
*** Lower bound to be covered ****
*********************************/


if dgp     == 1; true_lb = -b_cns*cdfn(0);  
elseif dgp == 2; true_lb = -b_cns*cdfn(-1);  
elseif dgp == 3; true_lb = -b_cns*cdfn(0);  
elseif dgp == 4; true_lb = -b_cns*cdfn(0);  

endif;
             
@               
" true_lb " true_lb;
@





/**************************
***    Simulation      ****
**************************/


    seed = 2345243;

    nrep = 1000;

  result_l = {};


i_rep = 1;
do until i_rep > nrep;

 
               v = 4*rndus(nn,1,seed)-2;  @ U[-2,2] @
               v = sortc(v,1);
               u = sm0*rndns(nn,1,seed); 
               
               u = abs(v).*u;        @ unequal variance @               
               
          
               u = -b_cns*(u .< -b_cns) + u.*(abs(u) .<= b_cns) + b_cns*(u .> b_cns);         
               
if     dgp == 1; fz0 = 0;  y = u;
elseif dgp == 2;

 fz0 = v.*(v .<= 1) + (v .> 1); 
   y = u;

elseif dgp == 3;

 fz0 = -2*(v.^2); 
   y = u;

elseif dgp == 4;

 fz0 = -10*(v.^2); 
   y = u;


endif;


               z = (fz0 + rndns(nn,1,seed) .> 0);  

/**
library pgraph;

if i_rep == 1;

if     dgp == 1; true_f = -b_cns*cdfn(0)*ones(rows(v),1);
elseif dgp == 2; true_f = -b_cns*cdfn(-fz0); 
elseif dgp == 3; true_f = -b_cns*cdfn(-fz0); 
endif;

     yl = y.*(z.== t) + k0*(z./= t);
  
_plctrl = {0,-1};
xy(v,true_f~yl);

endif;

**/




if cp_tht == 1;  @ coverage prob. with true_lb @

tht = true_lb;

elseif (cp_tht == 2) and (dgp == 1);

tht = true_lb - 0.03;

elseif (cp_tht == 2) and (dgp == 2);

tht = true_lb - 0.05;

elseif (cp_tht == 2) and (dgp == 3);

tht = true_lb - 0.05;

elseif (cp_tht == 2) and (dgp == 4);

tht = true_lb - 0.07;

endif;



{ts, pacv, gmscv} = as(y,z,v,t,v0,tht,0.05);



  result_l = result_l|(ts <= pacv)~(ts <= gmscv);


i_rep = i_rep + 1;
endo;


  

/**************************
*** Inference Results ****
**************************/

cov_acv = meanc(result_l);

cp_mat = cp_mat~cov_acv;



i_cp = i_cp + 1;
endo;

output on;
dgp~nn~cp_mat[1,.];
dgp~nn~cp_mat[2,.];
output off;


i_n = i_n + 1;
endo;

i_dgp = i_dgp + 1;
endo;



etime = ethsec(tstart,date)/6000;

output on;
" Running time   " etime;
"";
 output off; 
 
 
end;



/*********************************************************/
/*       Estimation and Inference for MM-MIV bounds      */
/*********************************************************/

proc(3) = as(y,z,v,t,v0,tht,level);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    z = n*1 vector of observed treatments
    v = n*1 vector of monotone instrumental variable
    t = 1*1 scalar of treatment of interest 
   v0 = 1*1 scalar of the point at which the bound is estimated 
  tht = a fixed parameter
  level = (1-level) confidence interval
--- Outputs ---
T_n(tht) in equation (3.14) of Andrews and Shi (2010)
c_n,1-alpha(tht) in equation2 (4.4) and (4.11) of  Andrews and Shi (2010) - PA and GMS critical values
@

local n,yl,mf,v_s,Tn,r,a_u,g_ar,mg_ar,mg_ar_mat,S_num,S_den,S,wr_mat,s_g1g2,D_tht,h2_g1g2,RR,rnd_RR,S0_num,S0_den,S0,T0,acv0,kappa_n,B_n,xi_n,h2bar_n,phi_n,S1_num,S1,T1,acv1;
  
@--- Calculating statistic: T_n(tht) ---@

        n = rows(y);
       yl = y.*(z.== t) + k0*(z./= t);
       mf = (v .<= v0).*(tht - yl);
      v_s = cdfn((v - meanc(v))/stdc(v));   @ studentization @


mg_ar_mat = {};  @ it will become an (n by sum_{r=1}^{r=7} 2r) matrix @
   wr_mat = {};  @ it will become a (sum_{r=1}^{r=7} 2r) vector @

 r = 1;

do until r > 7;

     a_u = seqa(1,1,2*r);
     g_ar = (v_s .> (a_u-1)'/(2*r)).*(v_s .<= a_u'/(2*r));   @ n by (2r) matrix @
    mg_ar = mf.*g_ar;   
mg_ar_mat = mg_ar_mat~mg_ar;   
   wr_mat = wr_mat|ones(2*r,1)/((2*r)*(r^2+100));
r = r + 1;
endo;


S_num = sqrt(n)*meanc(mg_ar_mat);
S_den = meanc((mg_ar_mat-meanc(mg_ar_mat)').^2) + (5/100)*meanc((mf-meanc(mf)').^2);
    S = S_num./sqrt(S_den);
    S = -S.*(S .< 0);  
    S = S.^2;

   Tn = sumc(S.*wr_mat);

@--- Calculating PA critical value ---@

 S_g1g2 = (mg_ar_mat-meanc(mg_ar_mat)')'(mg_ar_mat-meanc(mg_ar_mat)')/n;
  D_tht = (mf-meanc(mf)')'(mf-meanc(mf)')/n;
h2_g1g2 = S_g1g2/D_tht;

     RR = 5001;
 rnd_RR = chol(h2_g1g2 + 1e-8*eye(rows(h2_g1g2)))'rndns(cols(mg_ar_mat),RR,seed);
 
 S0_num = rnd_RR;
 S0_den = diag(h2_g1g2) + (5/100);
     S0 = S0_num./sqrt(S0_den);
     S0 = -S0.*(S0 .< 0);  
     S0 = S0.^2;

T0 = sumc(S0.*wr_mat);


 acv0 = quantile(T0,1-level);  @ PA critical values @  


@--- Calculating GMS critical value ---@

kappa_n = sqrt(0.3*ln(n));
    B_n = sqrt(0.4*ln(n)/ln(ln(n)));
   xi_n = S_num./sqrt(S_den);
h2bar_n = S_den/D_tht;
  phi_n = B_n*sqrt(h2bar_n).*(xi_n .> kappa_n);

 S1_num = rnd_RR + phi_n;
     S1 = S1_num./sqrt(S0_den);
     S1 = -S1.*(S1 .< 0);  
     S1 = S1.^2;

   T1 = sumc(S1.*wr_mat);
acv1 = quantile(T1,1-level);  @ GMS critical values @  

@--- collecting outputs ---@


retp(Tn,acv0,acv1);
endp;



