new;

/*************************************************
 Chernozhukov, Lee, and Rosen (2012)
 Monte Carlo Simulations for DGPs 9-12 in Supplementary Appendix L
 Nadaraya-Watson Kernel Estimation (*without* undersmoothing factor)
 
 Simulation design
 
 Y_i =  L*g(x) + min(3,max(-3,sigma*V)),   x \in [-2,2]
 V ~ N(0,1)

DGP 1: g(x) = phi(x^10), L = 1, sigma = 0.1
DGP 2: g(x) = phi(x^10), L = 5, sigma = 0.1
DGP 3: g(x) = max(phi((x-1.5)^10),phi((x+1.5)^10)), L = 1, sigma = 0.1
DGP 4: g(x) = max(phi((x-1.5)^10),phi((x+1.5)^10)), L = 5, sigma = 0.1

 \theta_0 = max L*\phi(x)

***************************************************/


output file = clr_mc_NW_Kernel_no_undersmooth_DGP_9_12.out reset;

output on;
"======== Monte Carlo Results ================================================";
"Column 1: Data generationg process ";
"Column 2: Sample size ";
"Column 3: argmin set estimation method ";
"Column 4: Coverage Probability (95%) ";
"Column 5: False Coverage Probability (95%) ";
"============================================================================";
output off;




format /rd /m1 8,3;

tstart = date;


dgp_set = {1,2,3,4};  
  n_set = {500,1000};
vse_set = {1,2};




i_dgp = 1;
do until i_dgp > rows(dgp_set);

i_n = 1;
do until i_n > rows(n_set); 

i_vse = 1;                      
do until i_vse > rows(vse_set); 




/**************************
*** Tuning Parameters  ****
**************************/

         nn = n_set[i_n];          @ sample size in simulations @
        dgp = dgp_set[i_dgp];      @ DGP @ 
     vsemtd = vse_set[i_vse];      @ method for argmin set estimation @ 
 
     b_cns = 3;



/*********************************
*** Lower bound to be covered ****
*********************************/


if dgp     == 1; true_lb = pdfn(0);  
elseif dgp == 2; true_lb = 5*pdfn(0);  
elseif dgp == 3; true_lb = pdfn(0);  
elseif dgp == 4; true_lb = 5*pdfn(0); 
endif;
             
@               
" true_lb " true_lb;
@




/**************************
***    Simulation      ****
**************************/




    seed = 2345243;


      RR = 10000;
   rnd_R = rndns(nn,RR,seed);   @ random numbers that will be used to callibrate critical values @
  



    nrep = 1000;

  result_l = {};
  h_matrix = {};
set_matrix = {};

i_rep = 1;
do until i_rep > nrep;


 
               v = 4*rndus(nn,1,seed)-2;  @ U[-2,2] @
               v = sortc(v,1);

if     dgp == 1;   sm0 = 0.1;  
elseif dgp == 2;   sm0 = 0.1;
elseif dgp == 3;   sm0 = 0.1;
elseif dgp == 4;   sm0 = 0.1;
endif;


               u = sm0*rndns(nn,1,seed); 

     
               u = -b_cns*(u .< -b_cns) + u.*(abs(u) .<= b_cns) + b_cns*(u .> b_cns);  

               
if     dgp == 1;   gx0 = pdfn(v^10);  
elseif dgp == 2;   gx0 = 5*pdfn(v^10);
elseif dgp == 3;   gx0 = maxc((pdfn((v-1.5)^10)~pdfn((v+1.5)^10))');
elseif dgp == 4;   gx0 = 5*maxc((pdfn((v-1.5)^10)~pdfn((v+1.5)^10))');
endif;


          y = gx0 + u;




               
@ bandwidth selection @
 
  vv = (v-meanc(v))/stdc(v);     @ studentization @ 

vreg = ones(nn,1)~vv~vv^2~vv^3~vv^4;
para = invpd(vreg'vreg)*(vreg'y);
resi = y - vreg*para;
sig2 = meanc(resi.^2);
drv2 = 2*para[3] + 6*para[4]*vv + 12*para[5]*vv^2;

 w0h = (vv .>= quantile(vv,0.1)).*(vv .<= quantile(vv,0.9));
 den = sumc((drv2.^2).*w0h);
 num = sig2*(maxc(w0h)-minc(w0h));         @ density weight between 0.1 and 0.9 quantiles @
   h = 2.036*stdc(v)*((num/den)^(1/5));    @ bandwidth for bound estimation @            
//   h = h*nn^(1/5)*nn^(-1/3.5);		   @ undersmooth @
hvar = h;                                  @ bandwidth for standard error estimation @   
  
h_matrix = h_matrix|h;      





@ argmax set estimation @

   sp_large = quantile(v,0.05)~quantile(v,0.95);    @ support of v for which the maximum is taken @

if vsemtd == 1; @ use the whole support @

      vseq = 100;
 argminset = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
      sp_l = sp_large;


elseif vsemtd == 2; @ use an estimated set @


@-- estimation of the argmin set --@

           vseq = 100;
       vs_large = seqa(sp_large[1],(sp_large[2]-sp_large[1])/vseq,vseq+1); @ grid points on the script V set @ 
         gamma_n = 0.1/ln(nn);

  {ml, sel, cvl} = kernel_est(y,v,vs_large,h,gamma_n); 

          cutoff = maxc(ml - sel*cvl) - 2*sel*cvl;
       argminset = selif(vs_large,(ml .>= cutoff));  @ estimated argminset @
            sp_l = minc(argminset)~maxc(argminset);

endif;


set_matrix = set_matrix|sp_l;


@--- Bounds estimatin ---@


levels = 0.50|0.05;

{ml2, sel2, cvl2} = kernel_est(y,v,argminset,h,levels);    

  l_b = maxc(ml2);

@--- Asymptotically median unbiased estimation  ---@

 ep_l = ml2 - sel2*cvl2[1];
 l_b1 = maxc(ep_l);


@--- Constructing confidence regions ---@

 ep_l = ml2 - sel2*cvl2[2];
 l_b2 = maxc(ep_l);


@--- collecting outputs ---@

lbb = l_b|l_b1|l_b2;

  result_l = result_l|lbb';






/**/
cov_l = meanc(result_l[.,3] .<= true_lb);

/**/

i_rep = i_rep + 1;
endo;


   
l1 = result_l[.,1];
l2 = result_l[.,2];
l3 = result_l[.,3];


      h_ave = meanc(h_matrix);
   sp_l_ave = meanc(set_matrix)';



/**************************
*** Inference Results ****
**************************/




if (dgp == 1);

false_lb = true_lb - 0.02;

elseif (dgp == 2);

false_lb = true_lb - 0.02;

elseif (dgp == 3);

false_lb = true_lb - 0.02;

elseif (dgp == 4);

false_lb = true_lb - 0.02;

endif;


 cov_cp = meanc(l3 .<= true_lb);
cov_fcp = meanc(l3 .<= false_lb);





output on;
dgp~nn~h_ave~vsemtd~cov_cp~cov_fcp~sp_l_ave;
output off;




i_vse = i_vse + 1;      
endo;  
                   

i_n = i_n + 1;
endo;

i_dgp = i_dgp + 1;
endo;


etime = ethsec(tstart,date)/6000;

output on;
" Running time   " etime;
"";
 output off; 
 
 
end;

/*********************************************************/
/*                Procedures                             */
/*********************************************************/

/*********************************************************/
/*           Kernel estimation                     */
/*********************************************************/

proc(3) = kernel_est(y,x,z,h,level);
@---
Input:
          y,x = data
            z = points at which a function is estimated 
            h = bandwidth   
        level = level of the confindence set

Output: function estimates, their standard errors, uniform critical values
---@

local n,ghat,fhat,m,v,s,tmp,norm,sehat,wn_x,tmp_RR,acv;

     n = rows(y);
  
@--- kernel estimation ---@

	ghat = nwk(y,x,z,h);          

@--- Pointwise standard error estimation ---@

    fhat = kde(x,z,h);           @ kernel density estimates @

       m = nwk(y,x,x,h);           
       v = y - m;
       s = kl((z'-x)/h);

     tmp = meanc((s.*v)^2)/h;    
     tmp = sqrt(tmp);

    norm = tmp./fhat;
   sehat = norm/sqrt(n*h);

 
@--- Critical values ---@

     wn_x = (((s'./fhat).*v')*rnd_R)/sqrt(n*h);    @ rnd_R is defined in the main program @ 

   tmp_RR = maxc(wn_x./norm);
      acv = quantile(tmp_RR,1-level);       @ critical values @ 

retp(ghat, sehat, acv);
endp;

/***

/*********************************************************/
/*  Estimation and Inference for Intersection bounds     */
/*********************************************************/

proc(2) = clr(y,v,h,hvar,sp_large,vsemtd);

local n,sp_l,vseq,vs_l,yl,ml,l_b,mll,vl,sl,fv_l,tmp_l,sel,tmp_RR,gamma_n,l_n,cutoff,argminset,wn_v,norm_l,level,acv,ep_l,l_b1,l_b2,lowerbound;

@--- Initial Tuning Parameters ---@

     n = rows(y);
  sp_l = sp_large;  @ support of v for lower bounds @

  vseq = 100;
  vs_l = seqa(sp_l[1],(sp_l[2]-sp_l[1])/vseq,vseq+1); @ evaluate the maximum at grid points @ 
  
@--- Calculating bounds ---@
  
ml = nwk(y,v,vs_l,h);           @ E[lower bound|v] @

l_b = maxc(ml);


@--- Set estimation ---@

    fv_l = kde(v,vs_l,h);           @ kernel density estimates @

     mll = nwk(y,v,v,h);           @ E[lower bound|v] @
      vl = y - mll;
      sl = kl((vs_l'-v)/h);

   tmp_l = meanc((sl.*vl)^2)/h;    
   tmp_l = sqrt(tmp_l);


  norm_l = tmp_l./fv_l;
     sel = norm_l/sqrt(n*h);

 
 if vsemtd == 1;

    sp_l = sp_large;

elseif vsemtd == 2;
 
    wn_v = (((sl'./fv_l).*vl')*rnd_R)/sqrt(n*h);    @ rnd_R is defined in the main program @    

  tmp_RR = maxc(wn_v)/sqrt(n*h);
 gamma_n = 0.1/ln(n);
     l_n = quantile(tmp_RR,1-gamma_n);   


   cutoff = maxc(ml - 3*l_n);
argminset = selif(vs_l,(ml .>= cutoff));
     sp_l = minc(argminset)~maxc(argminset);

endif;



@--- Critical values ---@

     vseq = 100;
     vs_l = seqa(sp_l[1],(sp_l[2]-sp_l[1])/vseq,vseq+1); @ re-define grid points @ 

     fv_l = kde(v,vs_l,h);           @ kernel density estimates @

      mll = nwk(y,v,v,h);           @ E[lower bound|v] @
       vl = y - mll;
       sl = kl((vs_l'-v)/h);

    tmp_l = meanc((sl.*vl)^2)/h;    
    tmp_l = sqrt(tmp_l);
   norm_l = tmp_l./fv_l;

     wn_v = (((sl'./fv_l).*vl')*rnd_R)/sqrt(n*h);    @ rnd_R is defined in the main program @ 
  
   tmp_RR = maxc(wn_v./norm_l);


    level = 0.50|0.05;
      acv = quantile(tmp_RR,1-level);       @ critical values @ 


 ep_l = ml - sel*acv[1];
 l_b1 = maxc(ep_l);


 ep_l = ml - sel*acv[2];
 l_b2 = maxc(ep_l);

@--- collecting outputs ---@

lowerbound = l_b|l_b1|l_b2;


retp(lowerbound,sp_l);

endp;

**/

   /*********************************************************/
   /*             Kernel Estimator                          */
   /*********************************************************/

proc(1) = nwk(y,x,x_arg,h);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of independent variables
x_arg = k*d vector of arguments of regression function
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel estimates
@

local n,d,k,i,arg,tmp1,tmp2,m,mhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

tmp1 = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
         tmp1 = tmp1.*kl(arg);

   i = i + 1;
   endo;

   m = y';
tmp2 = tmp1.*m;
mhat = sumc(tmp2')./(1e-8 + sumc(tmp1'));

retp(mhat);
endp;

/**********************************************************/
/*************** Kernel Density Estimator *****************/
/**********************************************************/
proc(1) = kde(x,x_arg,h);

@
--- Inputs ---
    x = n*d vector of observations
x_arg = k*d vector of arguments of density
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel density estimates
@

local n,d,k,i,arg,tmp,fhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

 tmp = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
          tmp = tmp.*(kl(arg)/h[i]);

   i = i + 1;
   endo;

fhat = meanc(tmp');

retp(fhat);
endp;

/*********************************************************/
/*       Local Linear Regression Estimator               */
/*********************************************************/

proc(1) = llre(y,x,x0,h);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*1 vector of observations of independent variables
   x0 = a vector of points where a regression function is evaluated
    h = a bandwidth
--- Output ---
local linear regression estimate (evaluated at x0)
@

local n,arg,w0,w1,w2,Sn0,Sn1,Sn2,v0,v1,Tn0,Tn1,tmp0,tmp1,tmp2,mhat,dmhat;

   n = rows(x);
  
 arg = (x-x0');
  w0 = kl(arg/h);
  w1 = w0.*arg;
  w2 = w0.*(arg.^2);
  
 Sn0 = meanc(w0);
 Sn1 = meanc(w1);
 Sn2 = meanc(w2);
 
  v0 = w0.*y;
  v1 = w1.*y;
  
 Tn0 = meanc(v0);
 Tn1 = meanc(v1);
 
  tmp0 = Sn2.*Sn0 - Sn1.^2;
  tmp0 = tmp0 + 1e-8*(tmp0.==0); 
  tmp1 = Sn2.*Tn0 - Sn1.*Tn1;
  tmp2 = Sn0.*Tn1 - Sn1.*Tn0;

  mhat = tmp1./tmp0;
 dmhat = tmp2./tmp0;
   
  

retp(mhat);


endp;


/**********************************************************/
/*************** 2nd Order Kernel Function ****************/
/**********************************************************/
proc(1) = kl(v);
local term;
term = (15/16)*((1-v^2)^2);
term = term.*(abs(v) .<= 1);
retp(term);
endp;



